/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef WWAKE_H
#define WWAKE_H

#include "dart.h"
#include "wGroups.h"
#include "wTag.h"
#include "wElement.h"
#include "wNode.h"
#include "wWakeElement.h"
#include <vector>
#include <string>

namespace dart
{

/**
 * @brief Group of elements to apply wake boundary conditions
 * @authors Adrien Crovato
 */
class DART_API Wake : public tbox::Groups
{
public:
    std::map<tbox::Node *, tbox::Node *> nodMap; ///< upper to lower surface nodes map
#ifndef SWIG
    std::vector<WakeElement *> wEle;
#endif

    Wake(std::shared_ptr<tbox::MshData> _msh, std::vector<int> const &nos);
    Wake(std::shared_ptr<tbox::MshData> _msh, std::vector<std::string> const &names);
    virtual ~Wake();

#ifndef SWIG
    virtual void write(std::ostream &out) const override;
#endif

private:
    void connectNodes();
    void createElements();
    void connectSurElements(std::map<tbox::Element *, tbox::Element *> &ssMap);
    void connectVolElements(tbox::Tag *tagS, tbox::Tag *tagV, std::map<tbox::Element *, tbox::Element *> &svMap);
    void findNodes(std::map<tbox::Element *, std::vector<std::pair<size_t, tbox::Node *>>> &mapEN);
};

} // namespace dart

#endif //WWAKE_H
